import javax.microedition.lcdui.*;

public class AllStationList extends List implements CommandListener {
    private Station[]  stations;
    
    public AllStationList() 
    {
        super(Locale.ALL_STATIONS, List.IMPLICIT);
        Line[] lines = Scheme.instance.metroLines;
        int nStations = 0;
        for (int i = 0; i < lines.length; i++) 
        { 
            nStations += lines[i].stations.length;
        }
        String[]  stationNames = new String[nStations];
        Image[]   stationIcons = new Image[nStations];
        stations = new Station[nStations];

        for (int i = 0, j = 0; i < lines.length; i++) { 
            for (int k = 0; k < lines[i].stations.length; k++, j++) { 
                stations[j] = lines[i].stations[k];
                stationNames[j] = stations[j].getShortName();
                stationIcons[j] = lines[i].image;
            }
        }
        sort(stations, stationNames, stationIcons);
        for (int i = 0; i < nStations; i++) { 
            append(stationNames[i], stationIcons[i]);
        }

        setCommandListener(this);
        addCommand(MetroNavigator.BACK_CMD);
        addCommand(MetroNavigator.SEARCH_CMD);
        addCommand(MetroNavigator.SELECT_CMD);
        Display.getDisplay(MetroNavigator.getInstance()).setCurrent(this);
    }
    
    public void commandAction(Command c, Displayable d)  {
        MetroNavigator navigator = MetroNavigator.getInstance();

        if (c == MetroNavigator.SEARCH_CMD) {
            new SearchBox(this);
        } else if (c == MetroNavigator.BACK_CMD) {
            Display.getDisplay(navigator).setCurrent(navigator.actionList);
        } else {               
            navigator.setStation(stations[getSelectedIndex()]);
            navigator.startApp();
        }
    }

    private static void sort(Station[] stations, String[] stationNames, Image[] stationIcons) {
        int base, last, first, middle; 
        first = middle = 1;
        last = stations.length - 1;
        while (first != last) {
            ++first;
            if (stationNames[middle].compareTo(stationNames[first]) > 0) {
                middle = first;
            }
        }
        if (stationNames[0].compareTo(stationNames[middle]) > 0) {
            Station tempStation = stations[0];
            stations[0] = stations[middle];
            stations[middle] = tempStation;
            String tempName = stationNames[0];
            stationNames[0] = stationNames[middle];
            stationNames[middle] = tempName;
            Image tempIcon = stationIcons[0];
            stationIcons[0] = stationIcons[middle];
            stationIcons[middle] = tempIcon;
        }
        base = 1;            
        while (base != last) {
            first = base++;
            if (stationNames[first].compareTo(stationNames[base]) > 0) {
                middle = base;
                Station tempStation = stations[middle];
                String tempName = stationNames[middle];
                Image tempIcon = stationIcons[middle];

                do {
                    stations[middle] = stations[first];
                    stationNames[middle] = stationNames[first];
                    stationIcons[middle] = stationIcons[first];
                    middle -= 1;
                    first -= 1;
                } while (stationNames[first].compareTo(tempName) > 0);

                stations[middle] = tempStation;
                stationNames[middle] = tempName;
                stationIcons[middle] = tempIcon;
            }
        }
    }
}
